package se.chalmers.tdv.hashtable;

/**
 * Data object describing the modifying actions of a hash table.
 */
public class LLHashTableAction {

	public enum ActionType {
		PUT, REMOVE, GET, OUTPUT
	};

	public final ActionType type;
	public final int key;
	public final Integer value;

	public LLHashTableAction(ActionType type, int key, Integer value) {
		this.type = type;
		this.key = key;
		this.value = value;
	}

	public String toString() {
		switch (this.type) {
		case PUT:
			return "put(" + this.key + ", " + this.value + ")";
		case REMOVE:
			return "remove(" + this.key + ")";
		case GET:
			return "get(" + this.key + ")";
		case OUTPUT:
			return "output(" + this.value + ")";
		default:
			throw new RuntimeException("Unknown action.");
		}
	}

	public static LLHashTableAction newPutAction(int key, int value) {
		return new LLHashTableAction(ActionType.PUT, key, value);
	}

	public static LLHashTableAction newRemoveAction(int key) {
		return new LLHashTableAction(ActionType.REMOVE, key, 0);
	}

	public static LLHashTableAction newGetAction(int key) {
		return new LLHashTableAction(ActionType.GET, key, 0);
	}

	public static LLHashTableAction newOutputAction(Integer value) {
		return new LLHashTableAction(ActionType.OUTPUT, 0, value);
	}

}
